# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import admin, status, parsers, queries, documents, collections
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, ZeroEntropyError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "ZeroEntropy",
    "AsyncZeroEntropy",
    "Client",
    "AsyncClient",
]


class ZeroEntropy(SyncAPIClient):
    admin: admin.AdminResource
    status: status.StatusResource
    collections: collections.CollectionsResource
    documents: documents.DocumentsResource
    queries: queries.QueriesResource
    parsers: parsers.ParsersResource
    with_raw_response: ZeroEntropyWithRawResponse
    with_streaming_response: ZeroEntropyWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous ZeroEntropy client instance.

        This automatically infers the `api_key` argument from the `ZEROENTROPY_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ZEROENTROPY_API_KEY")
        if api_key is None:
            raise ZeroEntropyError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ZEROENTROPY_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ZEROENTROPY_BASE_URL")
        if base_url is None:
            base_url = f"https://api.zeroentropy.dev/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.admin = admin.AdminResource(self)
        self.status = status.StatusResource(self)
        self.collections = collections.CollectionsResource(self)
        self.documents = documents.DocumentsResource(self)
        self.queries = queries.QueriesResource(self)
        self.parsers = parsers.ParsersResource(self)
        self.with_raw_response = ZeroEntropyWithRawResponse(self)
        self.with_streaming_response = ZeroEntropyWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncZeroEntropy(AsyncAPIClient):
    admin: admin.AsyncAdminResource
    status: status.AsyncStatusResource
    collections: collections.AsyncCollectionsResource
    documents: documents.AsyncDocumentsResource
    queries: queries.AsyncQueriesResource
    parsers: parsers.AsyncParsersResource
    with_raw_response: AsyncZeroEntropyWithRawResponse
    with_streaming_response: AsyncZeroEntropyWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncZeroEntropy client instance.

        This automatically infers the `api_key` argument from the `ZEROENTROPY_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ZEROENTROPY_API_KEY")
        if api_key is None:
            raise ZeroEntropyError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ZEROENTROPY_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ZEROENTROPY_BASE_URL")
        if base_url is None:
            base_url = f"https://api.zeroentropy.dev/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.admin = admin.AsyncAdminResource(self)
        self.status = status.AsyncStatusResource(self)
        self.collections = collections.AsyncCollectionsResource(self)
        self.documents = documents.AsyncDocumentsResource(self)
        self.queries = queries.AsyncQueriesResource(self)
        self.parsers = parsers.AsyncParsersResource(self)
        self.with_raw_response = AsyncZeroEntropyWithRawResponse(self)
        self.with_streaming_response = AsyncZeroEntropyWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class ZeroEntropyWithRawResponse:
    def __init__(self, client: ZeroEntropy) -> None:
        self.admin = admin.AdminResourceWithRawResponse(client.admin)
        self.status = status.StatusResourceWithRawResponse(client.status)
        self.collections = collections.CollectionsResourceWithRawResponse(client.collections)
        self.documents = documents.DocumentsResourceWithRawResponse(client.documents)
        self.queries = queries.QueriesResourceWithRawResponse(client.queries)
        self.parsers = parsers.ParsersResourceWithRawResponse(client.parsers)


class AsyncZeroEntropyWithRawResponse:
    def __init__(self, client: AsyncZeroEntropy) -> None:
        self.admin = admin.AsyncAdminResourceWithRawResponse(client.admin)
        self.status = status.AsyncStatusResourceWithRawResponse(client.status)
        self.collections = collections.AsyncCollectionsResourceWithRawResponse(client.collections)
        self.documents = documents.AsyncDocumentsResourceWithRawResponse(client.documents)
        self.queries = queries.AsyncQueriesResourceWithRawResponse(client.queries)
        self.parsers = parsers.AsyncParsersResourceWithRawResponse(client.parsers)


class ZeroEntropyWithStreamedResponse:
    def __init__(self, client: ZeroEntropy) -> None:
        self.admin = admin.AdminResourceWithStreamingResponse(client.admin)
        self.status = status.StatusResourceWithStreamingResponse(client.status)
        self.collections = collections.CollectionsResourceWithStreamingResponse(client.collections)
        self.documents = documents.DocumentsResourceWithStreamingResponse(client.documents)
        self.queries = queries.QueriesResourceWithStreamingResponse(client.queries)
        self.parsers = parsers.ParsersResourceWithStreamingResponse(client.parsers)


class AsyncZeroEntropyWithStreamedResponse:
    def __init__(self, client: AsyncZeroEntropy) -> None:
        self.admin = admin.AsyncAdminResourceWithStreamingResponse(client.admin)
        self.status = status.AsyncStatusResourceWithStreamingResponse(client.status)
        self.collections = collections.AsyncCollectionsResourceWithStreamingResponse(client.collections)
        self.documents = documents.AsyncDocumentsResourceWithStreamingResponse(client.documents)
        self.queries = queries.AsyncQueriesResourceWithStreamingResponse(client.queries)
        self.parsers = parsers.AsyncParsersResourceWithStreamingResponse(client.parsers)


Client = ZeroEntropy

AsyncClient = AsyncZeroEntropy
